/*
 * PolyStar
 * V1.0, October 2015
 * 
 * C. Ian Short
 * Saint Mary's University
 * Department of Astronomy and Physics
 * Institute for Computational Astrophysics (ICA)
 * Halifax, NS, Canada
 *  * ian.short@smu.ca
 * www.ap.smu.ca/~ishort/
 *
 * 1D, static, TE, polytropic interior structure model
 *
 * Suitable for pedagogical purposes only
 * 
 * Logic written in Java SE 8.0, JDK 1.8
 * GUI written with JavaFX 8.0
 * 
 * Ported to JavaScript for deployment
 *
 * System requirements for Java version: Java run-time environment (JRE)
 * System requirements for JavaScript version: JavaScript intrepretation enabld in WWW browser (usually by default)
 *
 * Code provided "as is" - there is no formal support 
 *
 * Java default license applies:
 * End users may adapt, modify, develop, debug, and deploy at will
 * for academic and othe non-profit uses, but are asked to leave this
 * header text in place (although they may add to the header text).
 *
 */


// **********************************************

"use strict";  //testing only!

// Global variables - Doesn't work - scope not global!

var c = 2.9979249E+10; // light speed in vaccuum in cm/s
var sigma = 5.670373E-5; //Stefan-Boltzmann constant ergs/s/cm^2/K^4  
var k = 1.3806488E-16; // Boltzmann constant in ergs/K
var h = 6.62606957E-27; //Planck's constant in ergs sec
var ee = 4.80320425E-10; //fundamental charge unit in statcoulombs (cgs)
var mE = 9.10938291E-28; //electron mass (g)
//Conversion factors
var amu = 1.66053892E-24; // atomic mass unit in g
var eV = 1.602176565E-12; // eV in ergs

//Methods:
//Natural logs more useful than base 10 logs - Eg. Formal soln module: 
// Fundamental constants
var logC = Math.log(c);
var logSigma = Math.log(sigma);
var logK = Math.log(k);
var logH = Math.log(h);
var logEe = Math.log(ee); //Named so won't clash with log_10(e)
var logMe = Math.log(mE);
//Conversion factors
var logAmu = Math.log(amu);
var logEv = Math.log(eV);

var logAStef = Math.log(4.0) + logSigma - logC;
var aStef = Math.exp(logAStef);
// ********************************************
// 
// 
// Utility functions:
//
//
// ********************************************



// logTen function (JS only provides natural log as standard on all browsers)

var logTen = function(x) {

// JS' 'log()' is really ln()

    return Math.log(x) / Math.LN10;
} //end logTen


// numToPxStng function to convert operable and calculable numbers into strings in 'px' for setting HTML style attributes

var numToPxStrng = function(x) {

    var xStr = x.toString(10) + "px"; // argument means interpret x as base 10 number

    return xStr;
} //end numToPxStrng


/*
 Linear interpolation to a new abscissa - mainly for interpolating flux to a specific lambda
 */

var interpol = function(x, y, newX) {

    var newY;
    var num = x.length;
    // Bracket newX:
    var x1, x2;
    var p1, p2;
    p1 = 0;
    p2 = 1;
    x1 = x[p1];
    x2 = x[p2];
    //console.log("num " + num + " initial x1 " + " x2 " + x2);
    for (var i = 1; i < num; i++) {

        if (x[0] < x[num - 1]) {
            // Case of monotonically increasing absicissae
            if (x[i] >= newX) {
// Found upper bracket
                p2 = i;
                p1 = i - 1;
                x2 = x[p2];
                x1 = x[p1];
                break;
            }
        } else {
            // Case of monotonically decreasing absicissae
            if (x[i] <= newX) {
// Found upper bracket
                p2 = i;
                p1 = i - 1;
                x2 = x[p2];
                x1 = x[p1];
                break;
            }
        }

    }

    var step = x2 - x1;
    //Interpolate
    newY = y[p2] * (newX - x1) / step
            + y[p1] * (x2 - newX) / step;
    //console.log("step " + step + " newX " + newX + " y[p2] " + y[p2] + " y[p1] " + y[p1] + " newY " + newY);
    //System.out.println("Interpol: p1, p2, x1, x2, y1, y2, newX, newY: " +
    //        p1 + " " + p2 + " " + x1 + " " + x2 + " " + y[1][p1] + " " + y[1][p2] + " " + newX + " " + newY + " ");
    //numPrint(roundNum, 100, 300, masterId); //debug

    return newY;
};

/*
 Linear interpolation to a new abscissa - mainly for interpoalting flux to a specific lambda
 This version for 2XN vector where we want to interpolate in row 1 - log units
 */

var interpol2 = function(x, y, newX) {

    var newY;
    // Bracket newX:
    var x1, x2;
    var p1, p2;
    p1 = 0;
    p2 = 1;
    x1 = x[p1];
    x2 = x[p2];
    for (var i = 1; i < x.length; i++) {
        if (x[i] >= newX) {
// Found upper bracket
            p2 = i;
            p1 = i - 1;
            x2 = x[p2];
            x1 = x[p1];
            break;
        }
    }

    var step = x2 - x1;
    //Interpolate
    // y is probably flux - Row 1 is log flux - let's interpolate in log space
    newY = y[1][p2] * (newX - x1) / step
            + y[1][p1] * (x2 - newX) / step;
    //System.out.println("Interpol: p1, p2, x1, x2, y1, y2, newX, newY: " + 
    //        p1 + " " + p2 + " " + x1 + " " + x2 + " " + y[1][p1] + " " + y[1][p2] + " " + newX + " " + newY + " ");
    //numPrint(roundNum, 100, 300, masterId); //debug

    return newY;
};

var lamPoint = function(numLams, lambdas, lam) {

    var index;

    var help = [];
    help.length = numLams;

    for (var i = 0; i < numLams; i++) {

        help[i] = lambdas[i] - lam;
        help[i] = Math.abs(help[i]);

    }
    index = 0;
    var min = help[index];

    for (var i = 1; i < numLams; i++) {

        if (help[i] < min) {
            min = help[i];
            index = i;
        }

    }

    return index;

};

/**
 * Return the array indices of minimum and maximum values of an input 1D array CAUTION; Will
 * return the *first* occurence if min and/or max values occur in multiple
 * places iMinMax[0] = first occurence of minimum iMinMax[1] = first occurence
 * of maximum
 */
var minMax = function(x) {

    var iMinMax = [];
    iMinMax.length = 2;

    var num = x.length;
    //System.out.println("MinMax: num: " + num);

    var iMin = 0;
    var iMax = 0;
    var min = x[iMin];
    var max = x[iMax];

    for (var i = 1; i < num; i++) {

        //System.out.println("MinMax: i , current min, x : " + i + " " + min + " " + x[i]);
        if (x[i] < min) {
            //System.out.println("MinMax: new min: if branch triggered" );
            min = x[i];
            iMin = i;
        }
        //System.out.println("MinMax: new min: " + min);

        if (x[i] > max) {
            max = x[i];
            iMax = i;
        }

    }
    //System.out.println("MinMax: " + iMin + " " + iMax);

    iMinMax[0] = iMin;
    iMinMax[1] = iMax;

    return iMinMax;

};


/**
 * Version of MinMax.minMax for 2XnumDep & 2XnumLams arrays where row 0 is
 * linear and row 1 is logarithmic
 *
 * Return the array indices of the minimum and maximum values of an input 1D array CAUTION; Will
 * return the *first* occurence if min and/or max values occur in multiple
 * places iMinMax[0] = first occurence of minimum iMinMax[1] = first occurence
 * of maximum
 */
var minMax2 = function(x) {

    var iMinMax = [];
    iMinMax.length = 2;

    var num = x[0].length;

    var iMin = 0;
    var iMax = 0;

    // Search for minimum and maximum in row 0 - linear values:
    var min = x[0][iMin];
    var max = x[0][iMax];

    for (var i = 1; i < num; i++) {

        if (x[0][i] < min) {
            min = x[0][i];
            iMin = i;
        }

        if (x[0][i] > max) {
            max = x[0][i];
            iMax = i;
        }

    }

    iMinMax[0] = iMin;
    iMinMax[1] = iMax;

    return iMinMax;

};

    var numPrint = function(val, x, y, RGBHex, areaId) {

        var xStr = numToPxStrng(x);
        var yStr = numToPxStrng(y);
     //   var RGBHex = colHex(r255, g255, b255);
        var valStr = val.toString(10);
        var numId = document.createElement("p");
        numId.style.position = "absolute";
        numId.style.display = "block";
        numId.style.marginTop = yStr;
        numId.style.marginLeft = xStr;
        numId.style.color = RGBHex;
        numId.innerHTML = valStr;
        //masterId.appendChild(numId);
        areaId.appendChild(numId);
    }; // end numPrint

    var txtPrint = function(text, x, y, width, RGBHex, areaId) {

        var xStr = numToPxStrng(x);
        var yStr = numToPxStrng(y);
        var wStr = numToPxStrng(width);
       // var RGBHex = colHex(r255, g255, b255);
        var txtId = document.createElement("p");
        txtId.style.position = "absolute";
        txtId.style.display = "block";
        txtId.style.width = wStr;
        txtId.style.marginTop = yStr;
        txtId.style.marginLeft = xStr;
        txtId.style.color = RGBHex;
        txtId.innerHTML = text;
        //masterId.appendChild(numId);
        areaId.appendChild(txtId);
    }; // end txtPrint

    var txtPrintGrid = function(text, RGBHex, areaId) {

       // var RGBHex = colHex(r255, g255, b255);
        var txtId = document.createElement("p");
        txtId.style.color = RGBHex;
        txtId.innerHTML = text;
        //masterId.appendChild(numId);
        areaId.appendChild(txtId);
    }; // end txtPrint

    var txtSVGprint = function(text, x, y, RGBHex, cnvsId) {

// orient must be:   "" (horizontal) OR "tb" (vertical

        var thisText = document.createElementNS(xmlW3, 'text');
        thisText.setAttributeNS(null, 'x', x);
        thisText.setAttributeNS(null, 'y', y);
        //textText.setAttribute('id',('text'+(i).toString()));
        //thisText.setAttribute('style', "font-style:normal;font-size:12px;");
        //thisText.textContent = text;
        thisText.innerHTML= text;
        thisText.setAttributeNS(xmlns,xmlnsLink,xmlnsLink2);
        cnvsId.appendChild(thisText);

    };

  var colHex = function(r255, g255, b255) {



        var rr = Math.floor(r255); //MUST convert to integer
        var gg = Math.floor(g255); //MUST convert to integer
        var bb = Math.floor(b255); //MUST convert to integer

        var RGBHex = "rgb(";
        RGBHex = RGBHex.concat(rr);
        RGBHex = RGBHex.concat(",");
        RGBHex = RGBHex.concat(gg);
        RGBHex = RGBHex.concat(",");
        RGBHex = RGBHex.concat(bb);
        RGBHex = RGBHex.concat(")");
//////    var RGBHex = "rgb(60,80,120)";

        return RGBHex;
    };

    //
    //
    //   ********* standForm()
    //
    //
    //
    var standForm = function(x) {
        // Turn any old number into the nearest number in standard form with a whole number exponent
        //   and a mantissa rounded to the nearest canonical value appropriate for labeling a tick mark
        //
        var numParts = [2];
        var isNeg = false;
        if (x === 0.0) {
            numParts = [0.0, 0.0];
        } else {

            if (x < 0) {
                isNeg = true;
                x = -1.0 * x;
            }

            var b = logTen(x);
            var n = Math.floor(b);
            var a = x / Math.pow(10.0, n);
            if (isNeg === true) {
                a = -1.0 * a;
            }

            numParts[0] = a; //mantissa
            numParts[1] = n; //exponent
        }

        return numParts;
    };

/**
 *
 * @author Ian vectorized version of simple linear 1st order interpolation
 * // Caution: Assumes new and old abscissae are in monotonic increasing order
 */

   var interpolV = function(y, x, newX) {

        var num = x.length;
        if (num != y.length){
          //System.out.println("Toolbox.interpolV(): Old x and y must be same length");
        }
        var newNum = newX.length;
        //System.out.println("interpolV: newNum " + newNum + " num " + num);
        var newY = [];
        newY.length = newNum;

//Renormalize ordinates:

    var iMinAndMax = minMax(y);

    var norm = y[iMinAndMax[1]];
    //System.out.println("norm " + norm);
    var yNorm = [];
    yNorm.length = num;
    var newYNorm = [];
    newYNorm.length = newNum;
    for (var i = 0; i < num; i++){
      yNorm[i] = y[i] / norm;
    }

// Set any newX elements that are *less than* the first x element to th first
// x element - "0th order extrapolation"
//
        var start = 0;
         for (var i = 0; i < newNum; i++) {
            if (newX[i] <= x[1]){
              newYNorm[i] = yNorm[0];
              start++;
            }
            if (newX[i] > x[1]){
               break;
            }
         }
  //System.out.println("start " + start);
  //System.out.println("x[0] " + x[0] + " x[1] " + x[1] + " newX[start] " + newX[start]);
  var jWght, jm1Wght, denom;

  if (start < newNum-1){

        var j = 1; //initialize old abscissae index
        //outer loop over new abscissae
        for (var i = start; i < newNum; i++) {

            //System.out.println("i " + i + " j " + j);

// break out if current element newX is *greater* that last x element
            if ( (newX[i] > x[num-1]) || (j > (num-1)) ){
               break;
            }

            while (x[j] < newX[i]) {
                j++;
            }
            //System.out.println("i " + i + " newX[i] " + newX[i] + " j " + j + " x[j-1] " + x[j-1] + " x[j] " + x[j]);
            //1st order Lagrange method:
            jWght = newX[i] * (1.0 - (x[j-1]/newX[i])); //(newX[i]-x[j-1])
            jm1Wght = x[j] * (1.0 - (newX[i]/x[j])); //(x[j]-newX[i])
            denom = x[j] * (1.0 - (x[j-1]/x[j])); //(x[j]-x[j-1])
            jWght = jWght / denom;
            jm1Wght = jm1Wght / denom;
            //newYNorm[i] = (yNorm[j]*(newX[i]-x[j-1])) + (yNorm[j-1]*(x[j]-newX[i]));
            newYNorm[i] = (yNorm[j]*jWght) + (yNorm[j-1]*jm1Wght);
            //System.out.println("i " + i + " newYNorm[i] " + newYNorm[i] + " j " + j + " yNorm[j-1] " + yNorm[j-1] + " yNorm[j] " + yNorm[j]);
        }

    } //start condition

// Set any newX elements that are *greater than* the first x element to the last
// x element - "0th order extrapolation"
//
         for (var i = 0; i < newNum; i++) {
            if (newX[i] >= x[num-1]){
              newYNorm[i] = yNorm[num-1];
            }
         }

//Restore orinate scale
    for (var i = 0; i < newNum; i++){
      newY[i] = newYNorm[i] * norm;
    }


        return newY;
 };


// Convert log temperature to RGB color triplet values out of 255 (24 bit color):
//
//Adapted from https://tannerhelland.com/2012/09/18/convert-temperature-rgb-algorithm-code.html
//How to Convert Temperature (K) to RGB: Algorithm and Sample Code
// Sep 18, 2012 • by Tanner Helland
//
  var logTempToRGB = function(logTemp){

    var log10E = logTen(Math.E); // convert log_e to log_10
//   'Given a temperature (in Kelvin), estimate an RGB equivalent

    var tmpCalc =0.0; 
    var r = 0;
    var g = 0;
    var b = 0;

    var logTemp10 = log10E * logTemp;

//    'logTemp must fall between 4 and 9 K 
    if (logTemp10 < 6.0){
	  logTemp10 = 6.0;
    }
    if (logTemp10 > 7.5){
	  logTemp10 = 7.5;
    }
    
// Original procedure expects input T-related value to be in range 10 to 400:
 // We must re-scale:
	  //

    var tmpKelvin = 10.0 + ( ((logTemp10-6.0)/(7.5-6.0)) * (400.0 - 10.0) );
   
  //console.log("logTemp10 " + logTemp10 + " tmpKelvin " + tmpKelvin);

//    'First: red
    if (tmpKelvin <= 66.0){ 
        r = 255;
    } else {
//        'Note: the R-squared value for this approximation is .988
        tmpCalc = tmpKelvin - 60.0;
        tmpCalc = 329.698727446 * (tmpCalc**(-0.1332047592));
        r = tmpCalc;
        if (r < 0){
	    r = 0;
	}
        if (r > 255){
	    r = 255;
	}
    } 
    
//    'Second: green
    if (tmpKelvin <= 66.0){ 
//        'Note: the R-squared value for this approximation is .996
        tmpCalc = tmpKelvin;
        tmpCalc = ( 99.4708025861 * Math.log(tmpCalc) ) - 161.1195681661;
        g = tmpCalc;
        if (g < 0){
            g = 0;
        }
        if (g > 255){
            g = 255;
        }

    } else {
//        'Note: the R-squared value for this approximation is .987
        tmpCalc = tmpKelvin - 60.0;
        tmpCalc = 288.1221695283 * (tmpCalc**(-0.0755148492));
        g = tmpCalc;
        if (g < 0){
            g = 0;
        }
        if (g > 255){
            g = 255;
        }

    }
    
//    'Third: blue
    if (tmpKelvin >= 66.0){ 
        b = 255;
    } else if (tmpKelvin <= 19.0){ 
        b = 0;
    } else { 
//        'Note: the R-squared value for this approximation is .998
        tmpCalc = tmpKelvin - 10.0;
        tmpCalc = (138.5177312231 * Math.log(tmpCalc)) - 305.0447927307;
        
        b = tmpCalc;
        if (b < 0){
            b = 0;
        }
        if (b > 255){
            b = 255;
        }

    } 

     //console.log("r " + r + " g " + g + " b " + b);
     var RGBHex = colHex(r, g, b);
   
     return RGBHex;

  };  //end logTempToRGB()

//
// Convert temperature to RGB color triplet values out of 255 (24 bit color):
//
//Adapted from https://tannerhelland.com/2012/09/18/convert-temperature-rgb-algorithm-code.html
//How to Convert Temperature (K) to RGB: Algorithm and Sample Code
// Sep 18, 2012 • by Tanner Helland
//
  var tempToRGB = function(temp){

//   'Given a temperature (in Kelvin), estimate an RGB equivalent

    var tmpCalc =0.0; 
    var r = 0;
    var g = 0;
    var b = 0;

//    'logTemp must fall between 10,000 and 1.0^9 K 
    if (temp < 1.0e4){
	  temp = 1.0e4;
    }
    if (temp > 1.0e9){
	  temp = 1.0e9;
    }
    
// Original procedure expects input T-related value to be in range 10 to 400:
 // We must re-scale:
	  //

    var tmpKelvin = 10.0 + ( ((temp-1.0e4)/(1.0e9-1.0e4)) * (400.0 - 10.0) );
    
//    'First: red
    if (tmpKelvin <= 66.0){ 
        r = 255;
    } else {
//        'Note: the R-squared value for this approximation is .988
        tmpCalc = tmpKelvin - 60.0;
        tmpCalc = 329.698727446 * (tmpCalc**(-0.1332047592));
        r = tmpCalc;
        if (r < 0){
	    r = 0;
	}
        if (r > 255){
	    r = 255;
	}
    } 
    
//    'Second: green
    if (tmpKelvin <= 66.0){ 
//        'Note: the R-squared value for this approximation is .996
        tmpCalc = tmpKelvin;
        tmpCalc = ( 99.4708025861 * Math.log(tmpCalc) ) - 161.1195681661;
        g = tmpCalc;
        if (g < 0){
            g = 0;
        }
        if (g > 255){
            g = 255;
        }

    } else {
//        'Note: the R-squared value for this approximation is .987
        tmpCalc = tmpKelvin - 60.0;
        tmpCalc = 288.1221695283 * (tmpCalc**(-0.0755148492));
        g = tmpCalc;
        if (g < 0){
            g = 0;
        }
        if (g > 255){
            g = 255;
        }

    }
    
//    'Third: blue
    if (tmpKelvin >= 66.0){ 
        b = 255;
    } else if (tmpKelvin <= 19.0){ 
        b = 0;
    } else { 
//        'Note: the R-squared value for this approximation is .998
        tmpCalc = tmpKelvin - 10.0;
        tmpCalc = (138.5177312231 * Math.log(tmpCalc)) - 305.0447927307;
        
        b = tmpCalc;
        if (b < 0){
            b = 0;
        }
        if (b > 255){
            b = 255;
        }

    } 

     var RGBHex = colHex(r, g, b);
   
     return RGBHex;

  };  //end tempToRGB()

//
//Input data on water phase in temp-press place
// - only boiling point significantly affected
//
   var waterPhase = function(atmPresIn){

//Input: Planetary atmospheric surface pressure in kPa 
//Ouput: boiling point in K

//Data from
//The Engineering ToolBox
//http://www.engineeringtoolbox.com/boiling-point-water-d_926.html
//Water  Pressure and Boiling Point
//Pressure Boiling Point
// kPa     deg C

  var numDataPnts = 99;
  var atmPresKPa = [];
  atmPresKPa.length = numDataPnts;
  var boilTempC = [];
  boilTempC.length = numDataPnts;

 atmPresKPa[0] = 3.45   ; boilTempC[0] = 26.4 ;
 atmPresKPa[1] = 6.90   ; boilTempC[1] = 38.7 ;
 atmPresKPa[2] = 13.79  ; boilTempC[2] = 52.2 ;
 atmPresKPa[3] = 20.69  ; boilTempC[3] = 60.8 ;
 atmPresKPa[4] = 27.58  ; boilTempC[4] = 67.2 ;
 atmPresKPa[5] = 34.48  ; boilTempC[5] = 72.3 ;
 atmPresKPa[6] = 41.37  ; boilTempC[6] = 76.7 ;
 atmPresKPa[7] = 48.27  ; boilTempC[7] = 80.4 ;
 atmPresKPa[8] = 55.16  ; boilTempC[8] = 83.8 ;
 atmPresKPa[9] = 62.06  ; boilTempC[9] = 86.8 ;
 atmPresKPa[10] = 68.95  ; boilTempC[10] = 89.6 ;
 atmPresKPa[11] = 75.85  ; boilTempC[11] = 92.1 ;
 atmPresKPa[12] = 82.74  ; boilTempC[12] = 94.4 ;
 atmPresKPa[13] = 89.64  ; boilTempC[13] = 96.6 ;
 atmPresKPa[14] = 96.53  ; boilTempC[14] = 98.7 ;
 atmPresKPa[15] = 101.3  ; boilTempC[15] = 100 ;
 atmPresKPa[16] = 103.4  ; boilTempC[16] = 101 ;
 atmPresKPa[17] = 110.3  ; boilTempC[17] = 102 ;
 atmPresKPa[18] = 117.2  ; boilTempC[18] = 104 ;
 atmPresKPa[19] = 124.1  ; boilTempC[19] = 106 ;
 atmPresKPa[20] = 131.0  ; boilTempC[20] = 107 ;
 atmPresKPa[21] = 137.9  ; boilTempC[21] = 109 ;
 atmPresKPa[22] = 151.7  ; boilTempC[22] = 112 ;
 atmPresKPa[23] = 165.5  ; boilTempC[23] = 114 ;
 atmPresKPa[24] = 179.3  ; boilTempC[24] = 117 ;
 atmPresKPa[25] = 193.1  ; boilTempC[25] = 119 ;
 atmPresKPa[26] = 206.9  ; boilTempC[26] = 121 ;
 atmPresKPa[27] = 220.6  ; boilTempC[27] = 123 ;
 atmPresKPa[28] = 234.4  ; boilTempC[28] = 125 ;
 atmPresKPa[29] = 248.2  ; boilTempC[29] = 127 ;
 atmPresKPa[30] = 262.0  ; boilTempC[30] = 129 ;
 atmPresKPa[31] = 275.8  ; boilTempC[31] = 131 ;
 atmPresKPa[32] = 289.6  ; boilTempC[32] = 132 ;
 atmPresKPa[33] = 303.4  ; boilTempC[33] = 134 ;
 atmPresKPa[34] = 317.2  ; boilTempC[34] = 135 ;
 atmPresKPa[35] = 331.0  ; boilTempC[35] = 137 ;
 atmPresKPa[36] = 344.8  ; boilTempC[36] = 138 ;
 atmPresKPa[37] = 358.5  ; boilTempC[37] = 140 ;
 atmPresKPa[38] = 372.3  ; boilTempC[38] = 141 ;
 atmPresKPa[39] = 386.1  ; boilTempC[39] = 142 ;
 atmPresKPa[40] = 399.9  ; boilTempC[40] = 144 ;
 atmPresKPa[41] = 413.7  ; boilTempC[41] = 145 ;
 atmPresKPa[42] = 427.5  ; boilTempC[42] = 146 ;
 atmPresKPa[43] = 441.3  ; boilTempC[43] = 147 ;
 atmPresKPa[44] = 455.1  ; boilTempC[44] = 148 ;
 atmPresKPa[45] = 468.9  ; boilTempC[45] = 149 ;
 atmPresKPa[46] = 482.7  ; boilTempC[46] = 151 ;
 atmPresKPa[47] = 496.4  ; boilTempC[47] = 152 ;
 atmPresKPa[48] = 510.2  ; boilTempC[48] = 153 ;
 atmPresKPa[49] = 524.0  ; boilTempC[49] = 154 ;
 atmPresKPa[50] = 537.8  ; boilTempC[50] = 155 ;
 atmPresKPa[51] = 551.6  ; boilTempC[51] = 156 ;
 atmPresKPa[52] = 565.4  ; boilTempC[52] = 157 ;
 atmPresKPa[53] = 579.2  ; boilTempC[53] = 158 ;
 atmPresKPa[54] = 593.0  ; boilTempC[54] = 158 ;
 atmPresKPa[55] = 606.8  ; boilTempC[55] = 159 ;
 atmPresKPa[56] = 620.6  ; boilTempC[56] = 160 ;
 atmPresKPa[57] = 634.3  ; boilTempC[57] = 161 ;
 atmPresKPa[58] = 648.1  ; boilTempC[58] = 162 ;
 atmPresKPa[59] = 661.9  ; boilTempC[59] = 163 ;
 atmPresKPa[60] = 675.7  ; boilTempC[60] = 164 ;
 atmPresKPa[61] = 689.5  ; boilTempC[61] = 164 ;
 atmPresKPa[62] = 724.0  ; boilTempC[62] = 166 ;
 atmPresKPa[63] = 758.5  ; boilTempC[63] = 168 ;
 atmPresKPa[64] = 792.9  ; boilTempC[64] = 170 ;
 atmPresKPa[65] = 827.4  ; boilTempC[65] = 172 ;
 atmPresKPa[66] = 1034   ; boilTempC[66] = 181 ;
 atmPresKPa[67] = 1207   ; boilTempC[67] = 189 ;
 atmPresKPa[68] = 1379   ; boilTempC[68] = 194 ;
 atmPresKPa[69] = 1551   ; boilTempC[69] = 200 ;
 atmPresKPa[70] = 1724   ; boilTempC[70] = 205 ;
 atmPresKPa[71] = 1896   ; boilTempC[71] = 210 ;
 atmPresKPa[72] = 2069   ; boilTempC[72] = 214 ;
 atmPresKPa[73] = 2241   ; boilTempC[73] = 218 ;
 atmPresKPa[74] = 2413   ; boilTempC[74] = 222 ;
 atmPresKPa[75] = 2586   ; boilTempC[75] = 226 ;
 atmPresKPa[76] = 2758   ; boilTempC[76] = 229 ;
 atmPresKPa[77] = 2930   ; boilTempC[77] = 233 ;
 atmPresKPa[78] = 3103   ; boilTempC[78] = 236 ;
 atmPresKPa[79] = 3275   ; boilTempC[79] = 239 ;
 atmPresKPa[80] = 3448   ; boilTempC[80] = 242 ;
 atmPresKPa[81] = 3620   ; boilTempC[81] = 245 ;
 atmPresKPa[82] = 3792   ; boilTempC[82] = 247 ;
 atmPresKPa[83] = 3965   ; boilTempC[83] = 250 ;
 atmPresKPa[84] = 4137   ; boilTempC[84] = 252 ;
 atmPresKPa[85] = 4309   ; boilTempC[85] = 255 ;
 atmPresKPa[86] = 4482   ; boilTempC[86] = 257 ;
 atmPresKPa[87] = 4654   ; boilTempC[87] = 260 ;
 atmPresKPa[88] = 4827   ; boilTempC[88] = 262 ;
 atmPresKPa[89] = 4999   ; boilTempC[89] = 264 ;
 atmPresKPa[90] = 5171   ; boilTempC[90] = 266 ;
 atmPresKPa[91] = 5344   ; boilTempC[91] = 268 ;
 atmPresKPa[92] = 5516   ; boilTempC[92] = 270 ;
 atmPresKPa[93] = 5688   ; boilTempC[93] = 272 ;
 atmPresKPa[94] = 5861   ; boilTempC[94] = 274 ;
 atmPresKPa[95] = 6033   ; boilTempC[95] = 276 ;
 atmPresKPa[96] = 6206   ; boilTempC[96] = 278 ;
 atmPresKPa[97] = 6550   ; boilTempC[97] = 281 ;
 atmPresKPa[98] = 6895   ; boilTempC[98] = 285 ;

    var steamTempK, steamTempC;  

    //steamTempC = interpolV(boilTempC, atmPresKPa, atmPresIn);
    steamTempC = interpol(atmPresKPa, boilTempC, atmPresIn);
    steamTempK = steamTempC + 273.0; 

    return steamTempK;  

   }; //end function waterPhase


//
//Input data on solvent phase in temp-press space
// - only boiling point significantly affected
//

//Input: Planetary atmospheric surface pressure in kPa
//     : Antoine coefficients for approximation to temperature-vapor pressure relation, A, B, C 
//Ouput: boiling point in K


   var solventPhase = function(atmPresIn, A, B, C){

      //var pTorr = atmPresIn / 0.133322; //kPa to torr 
      var pBar = 0.01 * atmPresIn; //kPa to bar 

      //var steamTempC = ( B / (A - logTen(pBar) ) ) - C;    
      var steamTempK = ( B / (A - logTen(pBar) ) ) - C;    

      //var steamTempK = steamTempC + 273.0;

      return steamTempK;  

   }; //end function solventPhase


//

var makeSpectralClass = function(teff, logg10){

    // Echo back the *actual* input parameters:
    //Add subclass to each spectral class (lburns)
    var spectralClass = " ";
    var subClass = " ";	//Create a variable for the subclass of the star. lburns
    var luminClass = "V";//defaults to V
//Determine the spectralClass and subClass of main sequence stars, subdwarfs and white dwarfs
//var luminClass = "V" or luminClass = "VI" or luminClass = "WD"
//#// Based on the data in Appendix G of An Introduction to Modern Astrophysics, 2nd Ed. by
//#// Carroll & Ostlie
if (((logg10 >= 4.0) && (logg10 < 5.0)) || ((logg10 >= 5.0) && (logg10 < 6.0)) || (logg10 >= 5.0)) {
    if (teff < 3000.0) {
        spectralClass = "L";
    } else if ((teff >= 3000.0) && (teff < 3900.0)) {
        spectralClass = "M";
	if ((teff >= 3000.0) && (teff <= 3030.0)) {
	    subClass = "6";
	} else if ((teff > 3030.0) && (teff <= 3170.0)) {
	    subClass = "5";
	} else if ((teff > 3170.0) && (teff <= 3290.0)) {
	    subClass = "4";
	} else if ((teff > 3290.0) && (teff <= 3400.0)) {
	    subClass = "3";
	} else if ((teff > 3400.0) && (teff <= 3520.0)) {
	    subClass = "2";
	} else if ((teff > 3520.0) && (teff <= 3660.0)) {
	    subClass = "1";
	} else if ((teff > 3660.0) && (teff < 3900.0)) {
	    subClass = "0";
	}
    } else if ((teff >= 3900.0) && (teff < 5200.0)) {
        spectralClass = "K";
	if ((teff >= 3900.0) && (teff <= 4150.0)) {
	    subClass = "7";
	} else if ((teff > 4150.0) && (teff <= 4410.0)) {
	    subClass = "5";
	} else if ((teff > 4410.0) && (teff <= 4540.0)) {
	    subClass = "4";
	} else if ((teff > 4540.0) && (teff <= 4690.0)) {
	    subClass = "3";
	} else if ((teff > 4690.0) && (teff <= 4990.0)) {
	    subClass = "1";
	} else if ((teff > 4990.0) && (teff < 5200.0)) {
	    subClass = "0";
	}
    } else if ((teff >= 5200.0) && (teff < 5950.0)) {
        spectralClass = "G";
	if ((teff >= 5200.0) && (teff <= 5310.0)) {
	    subClass = "8";
	} else if ((teff > 5310.0) && (teff <= 5790.0)) {
	    subClass = "2";
	} else if ((teff > 5790.0) && (teff < 5950.0)) {
	    subClass = "0";
	}
    } else if ((teff >= 5950.0) && (teff < 7300.0)) {
        spectralClass = "F";
	if ((teff >= 5950.0) && (teff <= 6250.0)) {
	    subClass = "8";
	} else if ((teff > 6250.0) && (teff <= 6650.0)) {
	    subClass = "5";
	} else if ((teff > 6650.0) && (teff <= 7050.0)) {
	    subClass = "2";
	} else if ((teff > 7050.0) && (teff < 7300.0)) {
	    subClass = "0";
	}
    } else if ((teff >= 7300.0) && (teff < 9800.0)) {
        spectralClass = "A";
	if ((teff >= 7300.0) && (teff <= 7600.0)) {
	    subClass = "8";
	} else if ((teff > 7600.0) && (teff <= 8190.0)) {
	    subClass = "5";
	} else if ((teff > 8190.0) && (teff <= 9020.0)) {
	    subClass = "2";
	} else if ((teff > 9020.0) && (teff <= 9400.0)) {
	    subClass = "1";
	} else if ((teff > 9400.0) && (teff < 9800.0)) {
	    subClass = "0";
	}
    } else if ((teff >= 9800.0) && (teff < 30000.0)) {
        spectralClass = "B";
	if ((teff >= 9300.0) && (teff <= 10500.0)) {
	    subClass = "9";
	} else if ((teff > 10500.0) && (teff <= 11400.0)) {
	    subClass = "8";
	} else if ((teff > 11400.0) && (teff <= 12500.0)) {
	    subClass = "7";
	} else if ((teff > 12500.0) && (teff <= 13700.0)) {
	    subClass = "6";
	} else if ((teff > 13700.0) && (teff <= 15200.0)) {
	    subClass = "5";
	} else if ((teff > 15200.0) && (teff <= 18800.0)) {
	    subClass = "3";
	} else if ((teff > 18800.0) && (teff <= 20900.0)) {
	    subClass = "2";
	} else if ((teff > 20900.0) && (teff <= 25400.0)) {
	    subClass = "1";
	} else if ((teff > 25400.0) && (teff < 30000.0)) {
	    subClass = "0";
	}
    } else if (teff >= 30000.0) {
        spectralClass = "O";
	if ((teff >= 30000.0) && (teff <= 35800.0)) {
	    subClass = "8";
	} else if ((teff > 35800.0) && (teff <= 37500.0)) {
	    subClass = "7";
	} else if ((teff > 37500.0) && (teff <= 39500.0)) {
	    subClass = "6";
	} else if ((teff > 39500.0) && (teff <= 42000.0)) {
	    subClass = "5"; 
	} 
    }
}
//Determine the spectralClass and subClass of giants and subgiants. lburns
//var luminClass = "III" or luminClass = "IV"
if (((logg10 >= 1.5) && (logg10 < 3.0)) || ((logg10 >= 3.0) && (logg10 < 4.0))) { 
    if (teff < 3000.0) {
	spectralClass = "L";
	} else if ((teff >= 3000.0) && (teff < 3700.0))  {
	spectralClass = "M";
	if ((teff >= 3000.0) && (teff <= 3330.0)) {
	    subClass = "6";
	} else if ((teff > 3330.0) && (teff <= 3380.0)) {
	    subclass = "5";
	} else if ((teff > 3380.0) && (teff <= 3440.0)) {
	    subClass = "4";
	} else if ((teff > 3440.0) && (teff <= 3480.0)) {
	    subClass = "3";
	} else if ((teff > 3480.0) && (teff <= 3540.0)) {
	    subClass = "2";
	} else if ((teff > 3540.0) && (teff <= 3600.0)) {
	    subClass = "1";
	} else if ((teff > 3600.0) && (teff < 3700.0)) {
	    subClass = "0";
	}
    } else if ((teff >= 3700.0) && (teff < 4700.0)) {
	spectralClass = "K";
	if ((teff >= 3700.0) && (teff <= 3870.0)) {
	    subClass = "7";
	} else if ((teff > 3870.0) && (teff <= 4050.0)) {
	    subClass = "5";
	} else if ((teff > 4050.0) && (teff <= 4150.0)) {
	    subClass = "4";
	} else if ((teff > 4150.0) && (teff <= 4260.0)) {
	    subClass = "3";
	} else if ((teff > 4260.0) && (teff <= 4510.0)) {
	    subClass = "1";
	} else if ((teff > 4510.0) && (teff < 4700.0)) {
	    subClass = "0";
	}
    } else if ((teff >= 4700.0) && (teff < 5500.0)) {
	spectralClass = "G";
	if ((teff >= 4700.0) && (teff <= 4800.0)) {
	    subClass = "8";
	} else if ((teff > 4800.0) && (teff <= 5300.0)) {
	    subClass = "2";
	} else if ((teff > 5300.0) && (teff < 5500.0)) {
	    subClass = "0";
	}
    } else if ((teff >= 5500.0) && (teff < 7500.0)) {
	spectralClass = "F";
	if ((teff >= 5500.0) && (teff <= 6410.0)) {
	    subClass = "5";
	} else if ((teff > 6410.0) && (teff <= 7000.0)) {
	    subClass = "2";
	} else if ((teff > 7000.0) && (teff < 7500.0)) {
	    subClass = "0";
	}
    } else if ((teff >= 7500.0) && (teff < 10300.0)) {
	spectralClass = "A";
	if ((teff >= 7500.0) && (teff <= 7830.0)) {
	    subClass = "8";
	} else if ((teff > 7830.0) && (teff <= 8550.0)) {
	    subClass = "5";
	} else if ((teff > 8550.0) && (teff <= 9460.0)) {
	    subClass = "2";
	} else if ((teff > 9460.0) && (teff <= 9820.0)) {
	    subClass = "1";
	} else if ((teff > 9820.0) && (teff < 10300.0)) {
	    subClass = "0";
	}
    } else if ((teff >= 10300.0) && (teff < 29300.0)) {
	spectralClass = "B";
	if ((teff >= 10300.0) && (teff <= 10900.0)) {
	    subClass = "9";
	} else if ((teff > 10900.0) && (teff <= 11700.0)) {
	    subClass = "8";
	} else if ((teff > 11700.0) && (teff <= 12700.0)) {
	    subClass = "7";
	} else if ((teff > 12700.0) && (teff <= 13800.0)) {
	    subClass = "6";
	} else if ((teff > 13800.0) && (teff <= 15100.0)) {
	    subClass = "5";
	} else if ((teff > 15100.0) && (teff <= 18300.0)) {
	    subClass = "3";
	} else if ((teff > 18300.0) && (teff <= 20200.0)) {
	    subClass = "2";
	} else if ((teff > 20200.0) && (teff <= 24500.0)) {
	    subClass = "1";
	} else if ((teff > 24500.0) && (teff < 29300.0)) {
	    subClass = "0";
	}
    } else if ((teff >= 29300.0) && (teff < 40000.0)) {
	spectralClass = "O";
	if ((teff >= 29300.0) && (teff <= 35000.0)) {
	    subClass = "8";
	} else if ((teff > 35000.0) && (teff <= 36500.0)) {
	    subClass = "7";
	} else if ((teff > 36500.0) && (teff <= 37800.0)) {
	    subClass = "6";
	} else if ((teff > 37800.0) && (teff < 40000.0)) {
	    subClass = "5"; 
	}
    }
} 

//Determine the spectralClass and subClass of supergiants and bright giants. lburns
//var luminClass = "I" or luminClass = "II"
if (((logg10 >= 0.0) && (logg10 < 1.0)) || ((logg10 >= 1.0) && (logg10 < 1.5))) {
    if (teff < 2700.0) {
	spectralClass = "L";
	} else if ((teff >= 2700.0) && (teff < 3650.0)) {
	spectralClass = "M";
	if ((teff >= 2700.0) && (teff <= 2710.0)) {
	    subClass = "6";
	} else if ((teff > 2710.0) && (teff <= 2880.0)) {
	    subClass = "5";
	} else if ((teff > 2880.0) && (teff <= 3060.0)) {
	    subClass = "4";
	} else if ((teff > 3060.0) && (teff <= 3210.0)) {
	    subClass = "3";
	} else if ((teff > 3210.0) && (teff <= 3370.0)) {
	    subClass = "2";
	} else if ((teff > 3370.0) && (teff <= 3490.0)) {
	    subClass = "1";
	} else if ((teff > 3490.0) && (teff < 3650.0)) {
	    subClass = "0";
	}
    } else if ((teff >= 3650.0) && (teff < 4600.0)) {
	spectralClass = "K"; 
	if ((teff >= 3650.0) && (teff <= 3830.0)) {
	    subClass = "7";
	} else if ((teff > 3830.0) && (teff <= 3990.0)) {
	    subClass = "5";
	} else if ((teff > 3990.0) && (teff <= 4090.0)) {
	    subClass = "4";
	} else if ((teff > 4090.0) && (teff <= 4190.0)) {
	    subClass = "3";
	} else if ((teff > 4190.0) && (teff <= 4430.0)) {
	    subClass = "1";
	} else if ((teff > 4430.0) && (teff < 4600.0)) {
	    subClass = "0";
	}
    } else if ((teff >= 4600.0) && (teff < 5500.0)) {
	spectralClass = "G";
	if ((teff >= 4600.0) && (teff <= 4700.0)) {
	    subClass = "8";
	} else if ((teff > 4700.0) && (teff <= 5190.0)) {
	    subClass = "2";
	} else if ((teff > 5190.0) && (teff < 5500.0)) {
	    subClass = "0";
	}
    } else if ((teff >= 5500.0) && (teff < 7500.0)) {
	spectralClass = "F";
	if ((teff >= 5500.0) && (teff <= 5750.0)) {
	    subClass = "8";
	} else if ((teff > 5750.0) && (teff <= 6370.0)) {
	    subClass = "5";
	} else if ((teff > 6370.0) && (teff <= 7030.0)) {
	    subClass = "2";
	} else if ((teff > 7030.0) && (teff < 7500.0)) {
	    subClass = "0";
	}
    } else if ((teff >= 7500.0) && (teff < 10000.0)) {
	spectralClass = "A";
	if ((teff >= 7500.0) && (teff <= 7910.0)) {
	    subClass = "8";
	} else if ((teff > 7910.0) && (teff <= 8610.0)) {
	    subClass = "5";
	} else if ((teff > 8610.0) && (teff <= 9380.0)) {
	    subClass = "2";
	} else if ((teff > 9380.0) && (teff < 10000.0)) {
	    subClass = "0";
	}
    } else if ((teff >= 10000.0) && (teff < 27000.0)) {
	spectralClass = "B";
	if ((teff >= 10000.0) && (teff <= 10500.0)) {
	    subClass = "9";
	} else if ((teff > 10500.0) && (teff <= 11100.0)) {
	    subClass = "8";
	} else if ((teff > 11100.0) && (teff <= 11800.0)) {
	    subClass = "7";
	} else if ((teff > 11800.0) && (teff <= 12600.0)) {
	    subClass = "6";
	} else if ((teff > 12600.0) && (teff <= 13600.0)) {
	    subClass = "5";
	} else if ((teff > 13600.0) && (teff <= 16000.0)) {
	    subClass = "3";
	} else if ((teff > 16000.0) && (teff <= 17600.0)) {
	    subClass = "2";
	} else if ((teff > 17600.0) && (teff <= 21400.0)) {
	    subClass = "1";
	} else if ((teff > 21400.0) && (teff < 27000.0)) {
	    subClass = "0";
	}
    } else if ((teff >= 27000.0) && (teff < 42000.0)) {
	spectralClass = "O";
	if ((teff >= 27000.0) && (teff <= 34000.0)) {
	    subClass = "8";
	} else if ((teff > 34000.0) && (teff <= 36200.0)) {
	    subClass = "7";
	} else if ((teff > 36200.0) && (teff <= 38500.0)) {
	    subClass = "6";
	} else if ((teff > 38500.0) && (teff < 42000.0)) {
	    subClass = "5";
	}
    } 	     
}

//console.log("L class initially: " + luminClass + " logg10 " + logg10);
//Determine luminClass based on logg
    if ((logg10 >= 0.0) && (logg10 < 1.0)) {
        luminClass = "I";
    } else if ((logg10 >= 1.0) && (logg10 < 1.5)) {
        luminClass = "II";
    } else if ((logg10 >= 1.5) && (logg10 < 3.0)) {
        luminClass = "III";
    } else if ((logg10 >= 3.0) && (logg10 < 4.0)) {
        luminClass = "IV";
    } else if ((logg10 >= 4.0) && (logg10 < 5.0)) {
        luminClass = "V";
    } else if ((logg10 >= 5.0) && (logg10 < 6.0)) {
        luminClass = "VI";
    } else if ((logg10 >= 5.0)){
        luminClass = "WD";
    }

//console.log("L class finally: " + luminClass + " logg10 " + logg10);
    var spectralType = "<a href='https://en.wikipedia.org/wiki/Stellar_classification' title='MK Spectral type' target='_blank'>" +
            spectralClass + subClass +  " " + luminClass +
            "</a>";

 return(spectralType);

}; // end fn makeSpectralClass
